//----------------------------------------------------------------------------
// 
//	file: rklib.cc
// 
//	function: Runge-Kutta method libraries 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Wed Nov 24 17:44:25 CST 1999
// 
//	$Revision: $ $Date: $
//	$Log$
//
//----------------------------------------------------------------------------
#include <iostream.h>
#include <math.h>
#include <stdlib.h>

typedef float (*Func) (float, int, float *);

#define 	drand48()	(rand()/(float)RAND_MAX) 

//----------------------------------------------------------------------------
//	f() 
//	  - define the function here
//		t : time
//		n : number of variables
//		vec : vector of variables
//----------------------------------------------------------------------------
float E(float t) {
  return (-32.0-0.65*t);
} 
float R(float t) {
  return (14.0/(0.05 + exp(-t/3.0) + exp(-t/12.0)));
}
float fx(float t, int n, float *vec) {

  static int c;
  static float spike_t;
  float x = vec[0], y = vec[1], z=vec[2];
  static int spike;
  // return (vec[1] - vec[0]*vec[0]); 
  // return (vec[1]);
  // return -vec[0] + 10.0*vec[1];
  //return 1.4-x*x+0.3*y-x;
  //return 0.1*vec[0]*(1.0-vec[0]);
  //return y-(x*x*x/3.0-x);
  // return -10.0*x + 10.0*y;
  // return (1.28 * x*x + -0.3*y);
  // return 3.86*x*(1-x);

  // Horn
  return (-0.45*x + -0.09 + 0.35*y*x + y*0.5); 

  // Integrate and fire : leaky Rm = 10.0. I = 0.12. Vth = 1.0 : with spike
  // display
  // return (x>(1.0))?(vec[0]=((x>1.5)?(0.0):(2.0))):((sin(c/100.0)+0.2) - x/10.0);
  c ++;

  // Stevens & Zador
  // return (x>(1.0))	? (vec[0]=((x>1.5)?(0.0):(2.0))) 
  //return (x>(1.0))?(vec[0]=((x>1.5)?(0.0):(2.0))):(- (x - E(t))/R(t) + 1.12 )/10.0;

   /*
  if (x>1.0) {
	spike_t = t;
  }
  
  return (-(x-E(t-spike_t))/R(t-spike_t) + 1.1)/10.0;
  */
    
}
float fy(float t, int n, float *vec) {
 
  float x = vec[0], y = vec[1], z=vec[2];
  // return (-vec[0]);
  // return (-vec[0]);
  // return -10*vec[0] - vec[1];
  //return x-y;
  //return 0.0;
  //return -x;
  //return -x*z + 2.0*x - y;
  // return x;

  //return x;
  // horn: 
  return (-y+(((y-x)>0.0)?1.0:0.0));

  //return 0.0;

}

float fz(float t, int n, float *vec) {
  float x = vec[0], y = vec[1], z=vec[2];

  return 0.0;
  // return x*y-8.0/3.0*z;

}
//----------------------------------------------------------------------------
//	rk4_solve() 
//	  - Runge Kutta method of order 4
//----------------------------------------------------------------------------
void rk4_solve(float start, float end, int steps, int numFuncs, 
	float *init, Func *funcs) {

  // allocate mem
  float wvec[numFuncs];
  float wvecTemp[numFuncs];
  float wvecTemp2[numFuncs];
  float K1[numFuncs];
  float K2[numFuncs];
  float K3[numFuncs];
  float K4[numFuncs];

  // initialize
  float h = (end-start)/(float)steps;
  float t = start;
  for (int i=0; i<numFuncs; ++i) {
	wvec[i] = init[i];
  }
  
  // iterate and integrate
  for (int i=1; i<=steps; ++i) {
 
	for (int n=0; n<numFuncs; ++n) {
 	  K1[n] = h*(funcs[n])(t, numFuncs, wvec);
	  wvecTemp[n] = wvec[n]+K1[n]/2.0;
  	}
	for (int n=0; n<numFuncs; ++n) {
 	  K2[n] = h*(funcs[n])(t+h/2.0, numFuncs, wvecTemp);
	  wvecTemp2[n] = wvec[n]+K2[n]/2.0;
  	}
	for (int n=0; n<numFuncs; ++n) {
 	  K3[n] = h*(funcs[n])(t+h/2.0, numFuncs, wvecTemp2);
	  wvecTemp[n] = wvec[n]+K3[n];
  	}
	for (int n=0; n<numFuncs; ++n) {
 	  K4[n] = h*(funcs[n])(t+h, numFuncs, wvecTemp);
  	}

	cout << t << " ";
	for (int n=0; n<numFuncs; ++n) {
	  wvec[n] = wvec[n] + (K1[n]+2.0*K2[n]+2.0*K3[n]+K4[n])/6.0;
	  cout << wvec[n] << " ";
	  t = start + i*h;
 	}
   	cout << endl;
  }
}

//----------------------------------------------------------------------------
//	euler_solve() 
//	  - Euler solver
//----------------------------------------------------------------------------
void euler_solve(float start, float end, int steps, int numFuncs, 
	float *init, Func *funcs) {

  // allocate mem
  float wvec[numFuncs];
  float wvecTemp[numFuncs];
  float K1[numFuncs];
  float K2[numFuncs];
  float K3[numFuncs];
  float K4[numFuncs];

  // initialize
  float h = 0.1; // fixed : (end-start)/(float)steps;
  float t = start;
  for (int i=0; i<numFuncs; ++i) {
	wvec[i] = init[i];
  }
  
  // iterate and integrate
  for (int i=1; i<=steps; ++i) {
 
	cout << t << " ";
	for (int n=0; n<numFuncs; ++n) {
	  wvecTemp[n] = wvec[n] + h*(funcs[n])(t,numFuncs,wvec);
	  cout << wvecTemp[n] << " ";
	  t = start + i*h;
 	}
	for (int n=0; n<numFuncs; ++n) {
	  wvec[n] = wvecTemp[n];
	}
   	cout << endl;
  }
}


//----------------------------------------------------------------------------
//	discrete_solve() 
//	  - discrete dynamical system solver
//----------------------------------------------------------------------------
void discrete_solve(float start, float end, int steps, int numFuncs,
        float *init, Func *funcs) {

  float wvec[numFuncs];
  float wvecTemp[numFuncs];

  for (int i=0; i<numFuncs; ++i) {
	wvec[i] = init[i];
  }

  float t = start;
  for (int i=1; i<=steps; ++i) {
	cout << t << " ";
  	for (int n=0; n<numFuncs; ++n) {
		wvecTemp[n] = (funcs[n])(t,numFuncs,wvec);
		cout << wvecTemp[n] << " ";
		t ++;
	}
	for (int n=0; n<numFuncs; ++n) {
	  wvec[n] = wvecTemp[n];
	}
	cout << endl;

  }
}
	
  
//----------------------------------------------------------------------------
//	main() 
//	  - Main test driver
//----------------------------------------------------------------------------
int main(int argc, char *argv[]) {

  Func Funcs[10];

  Funcs[0] = fx;
  Funcs[1] = fy;
  Funcs[2] = fz;

  if (argc < 7) {
	cerr << "usage: rklib <start> <end> <steps> <x> <y> <z> E|R|D\n";
	cerr << " 	E : Euler method\n";
	cerr << "	R : Fourth order Runge-Kutta method\n";
	cerr << "	D : Discrete dynamical system solver\n";
	exit(0);
  }
  float start = atof(argv[1]);
  float end   = atof(argv[2]);
  int   steps = atoi(argv[3]);
  float x = atof(argv[4]);
  float y = atof(argv[5]);
  float z = atof(argv[6]);
  float vals[2] = { x, y };

  if (argc == 8) {
	switch (argv[7][0]) {
		case 'E' : 
  			euler_solve(start,end,steps,3,vals,Funcs);
			break;
		case 'R' : 
  			rk4_solve(start,end,steps,3,vals,Funcs);
			break;
		case 'D' : 
  			discrete_solve(start,end,steps,3,vals,Funcs);
			break;
	}
  }


}


