function [C] = empcaol_iter(Cinit,k,iter)
%[C] = empcaol_iter(Cinit,k,iter)
%
% EMPCA ONLINE ITERATIONS
%
% (re)fits the model 
%
%    data = Cx + gaussian noise 
%
% with EM using x of dimension k 
% Gets points one at a time ONLINE. Uses the function nextpoint.m.
%
% Inputs:  k    is dimension of latent variable space 
%               (# of principal components)
%          Cinit is the initial (current) guess for C
%          iters is the number of iterations of EM to run
%
% Outputs: C is a (re)estimate of the matrix C 
%             whose columns span the principal subspace 
%
% uses nextpoint(1); to reinitialize nextpoint each pass through the data
% uses nextpoint(0) to get successive datavectors
% NB: nextpoint should return data with the mean already subtracted out
%

% check sizes and stuff


p = nextpoint(1);

if(isempty(Cinit)) 
  C = rand(p,k); 
else
  assert(k==size(Cinit,2));
  assert(p==size(Cinit,1));
  C = Cinit;
end

% loop for iterations
for ii=1:iter
  nextpoint(1);       % reset nextpoint
  C = empcaol1(C);    % let's do it
end

