#!/usr/bin/wish
#----------------------------------------------------------------------------
# tclplot : tcl interface to gnuplot 
# author: Yoonsuck Choe <yschoe@cs.utexas.edu>
#----------------------------------------------------------------------------

#----------------------------------------------------------------------------
#	User Configuration
#----------------------------------------------------------------------------
global gl
global plotopts

set gl(plotmethod) "func"
set gl(TMP)	"/tmp"
set gl(style) "line"
set gl(title) "NOTITLE"


#----------------------------------------------------------------------------
# tclplotDraw: draw whatever
#----------------------------------------------------------------------------
proc tclplotDraw { } {

  global gl
  switch $gl(plotmethod) {
	{func} { tclplotfunc }
	{data} { tclplotdata }
  }

}

#----------------------------------------------------------------------------
# tclplotdata: draw data
#----------------------------------------------------------------------------
proc tclplotdata { } {

  global gl
  global plotopts
  global columntitle

  if {[string length $gl(expression)]<=0} {
	return;
  }

  . config -cursor watch
  update

  #----------------------------------------
  # 1. Run data command and save result in tmp file
  # 	- assumes that command generates stdout
  #----------------------------------------
  set cp [open "$gl(TMP)/tclplot.cmd" "w"]
  puts $cp $gl(expression)
  close $cp

  set pp [open "|sh $gl(TMP)/tclplot.cmd" "r"]
  set tp [open "$gl(TMP)/tclplot.dat" "w"]
  set line [gets $pp]
  puts $tp $line
  # determine number of columns to plot
  regsub -all (\ |\t)(\ |\t)* $line { } cooked
  regsub -all (\ |\t)(\ |\t)*$ $cooked {} doublecooked
  set buf [split $doublecooked]
  set numcols [expr [llength $buf]-1]
  for {set i 0} {$i<[expr $numcols]} {incr i} {
	set fname ".plotframe.right.titleframe$i"
	if {![winfo exists $fname]} {
	    set columntitle($i) ""
  	    frame $fname 
 	    pack $fname -side top -padx 5 -pady 5 
 	    label $fname.label -text $i 
	    pack $fname.label -side left
	    entry $fname.entry -textvariable columntitle($i) -width 10
	    pack $fname.entry 
	}
  }
  while { ![eof $pp] } {
  	set line [gets $pp]
	puts $tp $line
  }
  close $tp
  close $pp

  #----------------------------------------
  # 2. Create gnuplot plot file
  #----------------------------------------
  set fp [open "$gl(TMP)/tclplot.gnu" "w"]
  puts $fp "set term gif"
  foreach opt [array names plotopts] {
  	if {$plotopts($opt)=="N"} {
		puts $fp "set no$opt"
	} elseif {$plotopts($opt)=="Y"} {
		puts $fp "set $opt"
	} else {
		puts $fp "set $opt $plotopts($opt)"
	}
  }
  puts $fp "set title \"$gl(title)\""
  puts $fp "set output \"$gl(TMP)/tclplot.gif\""
  puts -nonewline $fp "plot ";
  for {set i 0} {$i<$numcols} {incr i} {
	if {[string length $columntitle($i)]>0} {
		set tstr "title \"$columntitle($i)\""
	} else {
		set tstr "title \"NOTAG\""
	}
  	puts -nonewline $fp " \"$gl(TMP)/tclplot.dat\" "
	puts -nonewline $fp " u 1:[expr $i+2] $tstr w $gl(style)"
	if {$i != [expr $numcols-1]} {
		puts -nonewline $fp ","
	} else {
		puts $fp ""
	}
  }
  puts $fp "quit"
  close $fp

  #----------------------------------------
  # 3. Run gnuplot to get image
  #----------------------------------------
  exec gnuplot $gl(TMP)/tclplot.gnu

  destroy PLOT
  image create photo PLOT -file "$gl(TMP)/tclplot.gif"

  . config -cursor left_ptr

}

#----------------------------------------------------------------------------
# tclplotfunc: draw regular functions
#----------------------------------------------------------------------------
proc tclplotfunc { } {

  global gl plotopts

  if {[string length $gl(expression)]<=0} {
	return;
  }
  . config -cursor watch
  update
  set fp [open "$gl(TMP)/tclplot.gnu" "w"]
  puts $fp "set term gif"
  foreach opt [array names plotopts] {
  	if {$plotopts($opt)=="N"} {
		puts $fp "set no$opt"
	} elseif {$plotopts($opt)=="Y"} {
		puts $fp "set $opt"
	} else {
		puts $fp "set $opt $plotopts($opt)"
	}
  }
  puts $fp "set title \"$gl(title)\""
  puts $fp "set output \"$gl(TMP)/tclplot.gif\""
  puts $fp "$gl(expression) w $gl(style) "
  puts $fp "quit"
  close $fp

  exec gnuplot $gl(TMP)/tclplot.gnu

  destroy PLOT
  image create photo PLOT -file "$gl(TMP)/tclplot.gif"

  . config -cursor left_ptr
}

#----------------------------------------------------------------------------
# Set up user interface
#----------------------------------------------------------------------------
image create photo PLOT -width 640 -height 480

#----------------------------------------
# -1. Process cmd line args
#----------------------------------------

#----------------------------------------
# 0. Left frame
#----------------------------------------
frame .titleframe ; pack .titleframe -side top
label .titleframe.label -text "Title:"
pack  .titleframe.label -side left -padx 5 -pady 5
entry .titleframe.entry -textvariable gl(title) -width 30; 
pack .titleframe.entry -side left
bind .titleframe.entry <KeyPress-Return> "tclplotDraw"

frame .plotframe ; pack .plotframe -side top

set leftframe ".plotframe.left"
frame $leftframe ; pack $leftframe -side left

set rightframe ".plotframe.right"
frame $rightframe -relief ridge -border 2; pack $rightframe -side left -padx 5 -pady 5 -fill y

#----------------------------------------
# 1. Plot frame
#----------------------------------------
set plotframe "$leftframe.plot"
frame $plotframe -border 2 -relief sunken; pack $plotframe -padx 10 -pady 10

label $plotframe.photo -image PLOT; pack $plotframe.photo

#----------------------------------------
# 2. Expression input frame
#----------------------------------------
set expframe "$leftframe.exp"
frame $expframe ; pack $expframe  -padx 12 -pady 12 -side top

set gl(expression) "splot sin(sqrt(x*x+y*y))"
label $expframe.label -text "Expression:" ; pack $expframe.label -side left
entry $expframe.expr -textvariable gl(expression) -width 50; 
pack $expframe.expr -side left
bind $expframe.expr <KeyPress-Return> "tclplotDraw"
button $expframe.clear -text "Clear" -command "set gl(expression) \"\""
pack  $expframe.clear -side left -padx 5
button $expframe.but -text "Go" -command "tclplotDraw" ; pack  $expframe.but -side left -padx 5
#----------------------------------------
# 3. Datafile or pipe
#----------------------------------------

#----------------------------------------
# 4. Options
#----------------------------------------
set optframe ".opt"
frame $optframe; pack $optframe -side top -padx 5 -pady 5


#--------------------
# 4.1 set options
#--------------------
set plotopts(hidden) "N"
set plotopts(view) "30,45,1"
set plotopts(isosamples) "50"

#--------------------
# 4.2 add interface for optional parameters
#--------------------
foreach opt [array names plotopts] {
  frame $optframe.${opt}frame ; pack $optframe.${opt}frame -side left -padx 5
  label $optframe.${opt}frame.label -text $opt
  pack $optframe.${opt}frame.label -side left
  # if it is a boolean option, make it a radiobox
  if {$plotopts($opt)=="N" || $plotopts($opt)=="Y"} {
	radiobutton $optframe.${opt}frame.rad0 -text "On" \
		-variable plotopts($opt) -value "Y" \
		-border 1 -relief ridge
	pack $optframe.${opt}frame.rad0 -side left;
	radiobutton $optframe.${opt}frame.rad1 -text "Off" \
		-variable plotopts($opt) -value "N" \
		-border 1 -relief ridge
	pack $optframe.${opt}frame.rad1 -side left;
        bind $optframe.${opt}frame.rad0 <ButtonRelease-1> "tclplotDraw"
        bind $optframe.${opt}frame.rad1 <ButtonRelease-1> "tclplotDraw"
  } else {
 	entry $optframe.${opt}frame.entry -text $opt \
		-textvariable plotopts($opt) \
		-width [expr [string length $plotopts($opt)]+2]
	pack $optframe.${opt}frame.entry -side left;
        bind $optframe.${opt}frame.entry <KeyPress-Return> "tclplotDraw"
  }
  pack $optframe.${opt}frame.label -side left
} 

#--------------------
# 4.3 plotting method : from data or from function 
#--------------------
frame $optframe.method; pack $optframe.method -side left -padx 5
label $optframe.method.label -text "method" ; 
pack $optframe.method.label -side left 
radiobutton $optframe.method.rad0 -text "func" \
	-variable gl(plotmethod) -value "func" \
	-border 1 -relief ridge
pack $optframe.method.rad0 -side left;
radiobutton $optframe.method.rad1 -text "data" \
	-variable gl(plotmethod) -value "data" \
	-border 1 -relief ridge
pack $optframe.method.rad1 -side left;

#--------------------
# 4.4 style selection
#--------------------
frame $optframe.style; pack $optframe.style -side left -padx 5
menubutton $optframe.style.menub -text "Style"  -direction below \
	-menu $optframe.style.menub.menu -border 2 -relief ridge
pack $optframe.style.menub

menu $optframe.style.menub.menu
$optframe.style.menub.menu add radiobutton -label line -variable gl(style) 
$optframe.style.menub.menu add radiobutton -label impulse -variable gl(style)
$optframe.style.menub.menu add radiobutton -label points -variable gl(style)
$optframe.style.menub.menu add radiobutton -label linespoints -variable gl(style)

#----------------------------------------
# 5. Buttons
#----------------------------------------
set butframe ".opt.but"
frame $butframe ; pack $butframe

button $butframe.quit -text "Quit" -command exit; pack $butframe.quit -padx 15

#----------------------------------------------------------------------------
# main : nothing's done here yet
#----------------------------------------------------------------------------
wm geometry . +0+0

